<?php
/**
 * Plugin Name: Easy Maintenance Mode
 * Plugin URI: https://teesharp.com/easy-maintenance-mode
 * Description: A lightweight plugin to display a custom maintenance page with a countdown timer.
 * Version: 1.0.0
 * Author: Teesharp Limited
 * Author URI: https://teesharp.com
 * License: GPL v2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: easy-maintenance-mode
 * Domain Path: /languages
 */

// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}

class Easy_Maintenance_Mode
{

    /**
     * Plugin activation
     */
    public static function activate()
    {
        // Set default options on activation
        $default_options = array(
            'enabled' => false,
            'title' => 'Site Under Maintenance',
            'message' => 'We are performing scheduled maintenance. We will be back online shortly!',
            'end_date' => date('Y-m-d', strtotime('+1 day')),
            'end_time' => '12:00',
            'background_color' => '#f5f5f5',
            'text_color' => '#333333',
            'exclude_ips' => '',
            'countdown_enabled' => true
        );

        add_option('easy_maintenance_mode_options', $default_options);
    }

    /**
     * Plugin deactivation
     */
    public static function deactivate()
    {
        // Nothing to do on deactivation
    }

    /**
     * Plugin uninstall
     */
    public static function uninstall()
    {
        delete_option('easy_maintenance_mode_options');
    }

    /**
     * Initialize the plugin
     */
    public function __construct()
    {
        // Register activation, deactivation, and uninstall hooks
        register_activation_hook(__FILE__, array('Easy_Maintenance_Mode', 'activate'));
        register_deactivation_hook(__FILE__, array('Easy_Maintenance_Mode', 'deactivate'));
        register_uninstall_hook(__FILE__, array('Easy_Maintenance_Mode', 'uninstall'));

        // Add admin menu
        add_action('admin_menu', array($this, 'add_admin_menu'));

        // Register settings
        add_action('admin_init', array($this, 'register_settings'));

        // Add maintenance mode functionality
        add_action('template_redirect', array($this, 'display_maintenance_page'));

        // Add admin bar notification when maintenance mode is active
        add_action('admin_bar_menu', array($this, 'admin_bar_notification'), 100);

        // Add settings link on plugin page
        add_filter('plugin_action_links_' . plugin_basename(__FILE__), array($this, 'add_settings_link'));
    }

    /**
     * Add plugin menu under Settings
     */
    public function add_admin_menu()
    {
        add_options_page(
            'Easy Maintenance Mode Settings',
            'Maintenance Mode',
            'manage_options',
            'easy-maintenance-mode',
            array($this, 'display_settings_page')
        );
    }

    /**
     * Register plugin settings
     */
    public function register_settings()
    {
        register_setting(
            'easy_maintenance_mode_settings',
            'easy_maintenance_mode_options',
            array($this, 'sanitize_options')
        );
    }

    /**
     * Sanitize options before saving
     *
     * @param array $input The unsanitized options
     * @return array The sanitized options
     */
    public function sanitize_options($input)
    {
        $sanitized_input = array();

        $sanitized_input['enabled'] = isset($input['enabled']) ? true : false;
        $sanitized_input['title'] = sanitize_text_field($input['title']);
        $sanitized_input['message'] = wp_kses_post($input['message']);
        $sanitized_input['end_date'] = sanitize_text_field($input['end_date']);
        $sanitized_input['end_time'] = sanitize_text_field($input['end_time']);
        $sanitized_input['background_color'] = sanitize_hex_color($input['background_color']);
        $sanitized_input['text_color'] = sanitize_hex_color($input['text_color']);
        $sanitized_input['exclude_ips'] = sanitize_textarea_field($input['exclude_ips']);
        $sanitized_input['countdown_enabled'] = isset($input['countdown_enabled']) ? true : false;

        return $sanitized_input;
    }

    /**
     * Add settings link on plugin page
     *
     * @param array $links Plugin action links
     * @return array Modified plugin action links
     */
    public function add_settings_link($links)
    {
        $settings_link = '<a href="options-general.php?page=easy-maintenance-mode">' . __('Settings', 'easy-maintenance-mode') . '</a>';
        array_unshift($links, $settings_link);
        return $links;
    }

    /**
     * Display the settings page
     */
    public function display_settings_page()
    {
        // Get options
        $options = get_option('easy_maintenance_mode_options');
        ?>
        <div class="wrap">
            <h1><?php echo esc_html(get_admin_page_title()); ?></h1>

            <form method="post" action="options.php">
                <?php settings_fields('easy_maintenance_mode_settings'); ?>

                <table class="form-table">
                    <tr>
                        <th scope="row"><?php _e('Enable Maintenance Mode', 'easy-maintenance-mode'); ?></th>
                        <td>
                            <label>
                                <input type="checkbox" name="easy_maintenance_mode_options[enabled]" <?php checked(isset($options['enabled']) ? $options['enabled'] : false); ?>>
                                <?php _e('Activate maintenance mode', 'easy-maintenance-mode'); ?>
                            </label>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row"><?php _e('Title', 'easy-maintenance-mode'); ?></th>
                        <td>
                            <input type="text" name="easy_maintenance_mode_options[title]"
                                value="<?php echo esc_attr($options['title']); ?>" class="regular-text">
                        </td>
                    </tr>
                    <tr>
                        <th scope="row"><?php _e('Message', 'easy-maintenance-mode'); ?></th>
                        <td>
                            <?php
                            wp_editor(
                                isset($options['message']) ? $options['message'] : '',
                                'easy_maintenance_mode_message',
                                array(
                                    'textarea_name' => 'easy_maintenance_mode_options[message]',
                                    'media_buttons' => false,
                                    'textarea_rows' => 5,
                                    'teeny' => true
                                )
                            );
                            ?>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row"><?php _e('End Date & Time', 'easy-maintenance-mode'); ?></th>
                        <td>
                            <input type="date" name="easy_maintenance_mode_options[end_date]"
                                value="<?php echo esc_attr($options['end_date']); ?>">
                            <input type="time" name="easy_maintenance_mode_options[end_time]"
                                value="<?php echo esc_attr($options['end_time']); ?>">
                            <p class="description"><?php _e('When maintenance mode will end', 'easy-maintenance-mode'); ?></p>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row"><?php _e('Show Countdown', 'easy-maintenance-mode'); ?></th>
                        <td>
                            <label>
                                <input type="checkbox" name="easy_maintenance_mode_options[countdown_enabled]" <?php checked(isset($options['countdown_enabled']) ? $options['countdown_enabled'] : true); ?>>
                                <?php _e('Display countdown timer on the maintenance page', 'easy-maintenance-mode'); ?>
                            </label>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row"><?php _e('Background Color', 'easy-maintenance-mode'); ?></th>
                        <td>
                            <input type="color" name="easy_maintenance_mode_options[background_color]"
                                value="<?php echo esc_attr($options['background_color']); ?>">
                        </td>
                    </tr>
                    <tr>
                        <th scope="row"><?php _e('Text Color', 'easy-maintenance-mode'); ?></th>
                        <td>
                            <input type="color" name="easy_maintenance_mode_options[text_color]"
                                value="<?php echo esc_attr($options['text_color']); ?>">
                        </td>
                    </tr>
                    <tr>
                        <th scope="row"><?php _e('Exclude IPs', 'easy-maintenance-mode'); ?></th>
                        <td>
                            <textarea name="easy_maintenance_mode_options[exclude_ips]" rows="3"
                                class="large-text"><?php echo esc_textarea($options['exclude_ips']); ?></textarea>
                            <p class="description">
                                <?php _e('Enter IP addresses to exclude from maintenance mode, one per line. Your current IP is:', 'easy-maintenance-mode'); ?>
                                <?php echo esc_html($_SERVER['REMOTE_ADDR']); ?>
                            </p>
                        </td>
                    </tr>
                </table>

                <?php submit_button(); ?>
            </form>
        </div>
        <?php
    }

    /**
     * Display the maintenance page
     */
    public function display_maintenance_page()
    {
        // Get options
        $options = get_option('easy_maintenance_mode_options');

        // If maintenance mode is not enabled, return
        if (!isset($options['enabled']) || !$options['enabled']) {
            return;
        }

        // Skip for logged-in administrators
        if (current_user_can('manage_options')) {
            return;
        }

        // Skip if the current IP is in the exclude list
        $current_ip = $_SERVER['REMOTE_ADDR'];
        $excluded_ips = isset($options['exclude_ips']) ? explode("\n", str_replace("\r", "", $options['exclude_ips'])) : array();
        $excluded_ips = array_map('trim', $excluded_ips);

        if (in_array($current_ip, $excluded_ips)) {
            return;
        }

        // Set response code to 503 Service Unavailable
        status_header(503);

        // Set retry-after header to the end date/time
        $end_datetime = strtotime($options['end_date'] . ' ' . $options['end_time']);
        if ($end_datetime > time()) {
            header('Retry-After: ' . date('D, d M Y H:i:s T', $end_datetime));
        }

        // Display maintenance page
        $this->render_maintenance_page($options);
        exit;
    }

    /**
     * Render the maintenance page HTML
     *
     * @param array $options Plugin options
     */
    private function render_maintenance_page($options)
    {
        $title = isset($options['title']) ? $options['title'] : 'Site Under Maintenance';
        $message = isset($options['message']) ? $options['message'] : 'We are performing scheduled maintenance. We will be back online shortly!';
        $background_color = isset($options['background_color']) ? $options['background_color'] : '#f5f5f5';
        $text_color = isset($options['text_color']) ? $options['text_color'] : '#333333';
        $countdown_enabled = isset($options['countdown_enabled']) ? $options['countdown_enabled'] : true;
        $end_datetime = strtotime($options['end_date'] . ' ' . $options['end_time']);

        // Calculate time difference in seconds
        $time_diff = $end_datetime - time();

        // If end time is in the past, set time_diff to 0
        if ($time_diff < 0) {
            $time_diff = 0;
        }

        ?>
        <!DOCTYPE html>
        <html <?php language_attributes(); ?>>

        <head>
            <meta charset="<?php bloginfo('charset'); ?>">
            <meta name="viewport" content="width=device-width, initial-scale=1">
            <title><?php echo esc_html($title); ?> - <?php bloginfo('name'); ?></title>
            <style>
                body {
                    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Oxygen-Sans, Ubuntu, Cantarell, "Helvetica Neue", sans-serif;
                    background-color:
                        <?php echo esc_attr($background_color); ?>
                    ;
                    color:
                        <?php echo esc_attr($text_color); ?>
                    ;
                    margin: 0;
                    padding: 0;
                    display: flex;
                    align-items: center;
                    justify-content: center;
                    min-height: 100vh;
                    text-align: center;
                }

                .maintenance-container {
                    max-width: 800px;
                    padding: 40px;
                    background-color: rgba(255, 255, 255, 0.9);
                    border-radius: 8px;
                    box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1);
                    margin: 20px;
                }

                h1 {
                    font-size: 2.5em;
                    margin-bottom: 20px;
                }

                .countdown {
                    display: flex;
                    justify-content: center;
                    margin: 30px 0;
                    flex-wrap: wrap;
                }

                .countdown-item {
                    margin: 0 15px;
                    min-width: 80px;
                }

                .countdown-number {
                    font-size: 2.5em;
                    font-weight: bold;
                    background-color:
                        <?php echo esc_attr($text_color); ?>
                    ;
                    color:
                        <?php echo esc_attr($background_color); ?>
                    ;
                    padding: 10px;
                    border-radius: 5px;
                    display: inline-block;
                    min-width: 60px;
                }

                .countdown-label {
                    font-size: 0.9em;
                    margin-top: 5px;
                    text-transform: uppercase;
                }
            </style>
        </head>

        <body>
            <div class="maintenance-container">
                <h1><?php echo esc_html($title); ?></h1>
                <div><?php echo wp_kses_post($message); ?></div>

                <?php if ($countdown_enabled && $time_diff > 0): ?>
                    <div class="countdown">
                        <div class="countdown-item">
                            <div id="countdown-days" class="countdown-number">00</div>
                            <div class="countdown-label">Days</div>
                        </div>
                        <div class="countdown-item">
                            <div id="countdown-hours" class="countdown-number">00</div>
                            <div class="countdown-label">Hours</div>
                        </div>
                        <div class="countdown-item">
                            <div id="countdown-minutes" class="countdown-number">00</div>
                            <div class="countdown-label">Minutes</div>
                        </div>
                        <div class="countdown-item">
                            <div id="countdown-seconds" class="countdown-number">00</div>
                            <div class="countdown-label">Seconds</div>
                        </div>
                    </div>

                    <script>
                        // Countdown timer
                        var countdownDate = <?php echo esc_js($end_datetime * 1000); ?>;

                        function updateCountdown() {
                            var now = new Date().getTime();
                            var distance = countdownDate - now;

                            if (distance < 0) {
                                document.getElementById("countdown-days").innerHTML = "00";
                                document.getElementById("countdown-hours").innerHTML = "00";
                                document.getElementById("countdown-minutes").innerHTML = "00";
                                document.getElementById("countdown-seconds").innerHTML = "00";
                                return;
                            }

                            var days = Math.floor(distance / (1000 * 60 * 60 * 24));
                            var hours = Math.floor((distance % (1000 * 60 * 60 * 24)) / (1000 * 60 * 60));
                            var minutes = Math.floor((distance % (1000 * 60 * 60)) / (1000 * 60));
                            var seconds = Math.floor((distance % (1000 * 60)) / 1000);

                            document.getElementById("countdown-days").innerHTML = days < 10 ? "0" + days : days;
                            document.getElementById("countdown-hours").innerHTML = hours < 10 ? "0" + hours : hours;
                            document.getElementById("countdown-minutes").innerHTML = minutes < 10 ? "0" + minutes : minutes;
                            document.getElementById("countdown-seconds").innerHTML = seconds < 10 ? "0" + seconds : seconds;
                        }

                        // Update the countdown every second
                        updateCountdown();
                        setInterval(updateCountdown, 1000);
                    </script>
                <?php endif; ?>
            </div>
        </body>

        </html>
        <?php
    }

    /**
     * Add notification to the admin bar when maintenance mode is active
     *
     * @param WP_Admin_Bar $wp_admin_bar WP_Admin_Bar instance
     */
    public function admin_bar_notification($wp_admin_bar)
    {
        // Get options
        $options = get_option('easy_maintenance_mode_options');

        // If maintenance mode is not enabled, return
        if (!isset($options['enabled']) || !$options['enabled']) {
            return;
        }

        // Add the main notification
        $wp_admin_bar->add_node(array(
            'id' => 'maintenance-mode-active',
            'title' => '<span style="color: #ff6b6b; font-weight: bold;">' . __('Maintenance Mode Active', 'easy-maintenance-mode') . '</span>',
            'href' => admin_url('options-general.php?page=easy-maintenance-mode')
        ));
    }
}

// Initialize the plugin
$easy_maintenance_mode = new Easy_Maintenance_Mode();